// Copyright 2021 Esri
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//   http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

import ArcGIS
import UIKit

class ViewController: UIViewController {
    @IBOutlet weak var sceneView: AGSSceneView!

    override func viewDidLoad() {
        super.viewDidLoad()
        setupScene()

        setupUI()

        setupAnalysisOverlay()

    }

    // MARK: Scene

    private func setupScene() {
        let scene: AGSScene = {
            let portal = AGSPortal.arcGISOnline(withLoginRequired: false)

            let item = AGSPortalItem(portal: portal, itemID: "7558ee942b2547019f66885c44d4f0b1")

            return AGSScene(item: item)
        }()
        sceneView.scene = scene

        sceneView.touchDelegate = self

    }

    // MARK: UI

    private func setupUI() {
        navigationItem.title = "Long press and drag."
        let clear = UIBarButtonItem(title: "Clear", style: .plain, target: self, action: #selector(clearScene))
        navigationItem.rightBarButtonItem = clear
    }

    @objc
    func clearScene(_ sender: UIBarButtonItem) {

        clearMeasurement()

    }

    // MARK: Distance Measurement Formatter

    private static let measurementFormatter: MeasurementFormatter = {
        let measurementFormatter = MeasurementFormatter()
        measurementFormatter.numberFormatter.minimumFractionDigits = 2
        measurementFormatter.numberFormatter.maximumFractionDigits = 2
        return measurementFormatter
    }()

    private func setTitleForDistanceMeasurements(direct: AGSDistance, horizontal: AGSDistance, vertical: AGSDistance) {
        DispatchQueue.main.async { [weak self] in
            guard let self = self else { return }
            if self.measurement.isVisible {
                self.navigationItem.title = String(
                    format: "Direct: %@ | Horizontal: %@ | Vertical: %@",
                    Self.measurementFormatter.string(from: Measurement(value: direct.value, unit: Unit(symbol: direct.unit.abbreviation))),
                    Self.measurementFormatter.string(from: Measurement(value: horizontal.value, unit: Unit(symbol: horizontal.unit.abbreviation))),
                    Self.measurementFormatter.string(from: Measurement(value: vertical.value, unit: Unit(symbol: vertical.unit.abbreviation)))
                )
            } else {
                self.navigationItem.title = "Long press and drag."
            }
        }
    }

    // MARK: Distance Measurement Analysis

    private lazy var measurement: AGSLocationDistanceMeasurement = {
        let point = AGSPointMake3D(0, 0, 0, 0, .wgs84())
        let measurement = AGSLocationDistanceMeasurement(startLocation: point, endLocation: point)
        measurement.measurementChangedHandler = setTitleForDistanceMeasurements
        measurement.isVisible = false
        measurement.unitSystem = .metric // .imperial
        return measurement
    }()

    private func setupAnalysisOverlay() {
        let analysisOverlay = AGSAnalysisOverlay()
        analysisOverlay.analyses.add(measurement)
        sceneView.analysisOverlays.add(analysisOverlay)
    }

    private func startDistanceMeasurement(point: AGSPoint) {
        measurement.isVisible = false
        measurement.startLocation = point
        measurement.endLocation = point
    }

    private func moveDistanceMeasurement(point: AGSPoint) {
        measurement.endLocation = point
        if !measurement.isVisible {
            measurement.isVisible = true
        }
    }

    private func clearMeasurement() {
        measurement.isVisible = false
        navigationItem.title = "Long press and drag."
    }

}

extension ViewController: AGSGeoViewTouchDelegate {

    func geoView(_ geoView: AGSGeoView, didLongPressAtScreenPoint screenPoint: CGPoint, mapPoint: AGSPoint) {

        startDistanceMeasurement(point: mapPoint)

    }

    func geoView(_ geoView: AGSGeoView, didMoveLongPressToScreenPoint screenPoint: CGPoint, mapPoint: AGSPoint) {

        moveDistanceMeasurement(point: mapPoint)

    }

    func geoView(_ geoView: AGSGeoView, didEndLongPressAtScreenPoint screenPoint: CGPoint, mapPoint: AGSPoint) {

        moveDistanceMeasurement(point: mapPoint)

    }

    func geoViewDidCancelLongPress(_ geoView: AGSGeoView) {

        clearMeasurement()

    }

}


